<?php
/**
 * @package     HSFirePHP!
 * @author      Hosting Skills a.s.b.l. - https://www.hosting-skills.lu
 * @copyright   Copyright (C) 2014-2021 Hosting Skills a.s.b.l.. All rights reserved.
 * @license     GNU/GPLv3 https://www.gnu.org/licenses/gpl-3.0.html
 */

defined('_JEXEC') or die;

use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;

/**
 * HSFirePHP! System Plugin Installer Script
 *
 * @since   1.0.0
 */
class plgSystemHSFirePHPInstallerScript {
	/**
	 * Version of the Plugin
	 *
	 * @var     string
	 *
	 * @since   1.0.0
	 */
	protected string $version;

	/**
	 * Function called before extension installation/update/removal procedure commences
	 *
	 * @param   string            $type    The type of change (install, update, discover_install or uninstall)
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.0.0
	 */
	public function preflight(string $type, InstallerAdapter $parent): bool
	{
		if ('uninstall' === $type)
		{
			return true;
		}

		$this->version  = $parent->getManifest()->version;
		$joomla_version = $parent->getManifest()->attributes()->version;
		$php_version    = $parent->getManifest()->attributes()->php;

		// Check the minimum php version.
		if (version_compare(PHP_VERSION, $php_version, 'lt'))
		{
			$parent->getParent()->message = '';
			Log::add(sprintf(Text::_('PLG_SYSTEM_HSFIREPHP_PHP_VERSION_ERROR'), $php_version), Log::WARNING, 'jerror');

			return false;
		}

		// Check the minimum Joomla! version.
		if (version_compare(JVERSION, $joomla_version, 'lt'))
		{
			$parent->getParent()->message = '';
			Log::add(sprintf(Text::_('PLG_SYSTEM_HSFIREPHP_JOOMLA_VERSION_ERROR'), $joomla_version), Log::WARNING, 'jerror');

			return false;
		}

		if ('update' === $type)
		{
			$db    = $parent->getParent()->getDbo();
			$query = $db->getQuery(true)
				->select($db->quoteName('manifest_cache'))
				->from($db->quoteName('#__extensions'))
				->where($db->quoteName('name') . ' = ' . $db->quote($parent->getManifest()->name));
			$db->setQuery($query);

			try
			{
				$db->execute();
				$manifest       = json_decode($db->loadResult(), true);
				$update_version = $manifest['version'];
			}
			catch (RuntimeException $e)
			{
				$update_version = $this->version;
			}

			// Check for real upgrade of extension.
			if (version_compare($this->version, $update_version, 'le'))
			{
				$parent->getParent()->message = '';
				Log::add(sprintf(Text::_('PLG_SYSTEM_HSFIREPHP_UPDATE_VERSION_ERROR'), $update_version, $this->version), Log::WARNING, 'jerror');

				return false;
			}
		}

		return true;
	}

	/**
	 * This method is called after a component is installed.
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.0.0
	 */
	public function install(InstallerAdapter $parent): bool
	{
		$parent->getParent()->message = sprintf(Text::_('PLG_SYSTEM_HSFIREPHP_INSTALL'), $this->version) . '<br /><br /><span style="color: red;">' . Text::_('PLG_SYSTEM_HSFIREPHP_CONFIGURE') . '</span><br /><br /><img src="../plugins/system/hsfirephp/hsfirephp.png" alt="HSFirePHP!" /><br /><br />';

		return true;
	}

	/**
	 * This method is called after a component is updated.
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.0.0
	 */
	public function update(InstallerAdapter $parent): bool
	{
		$db    = $parent->getParent()->getDbo();
		$query = $db->getQuery(true)
			->select($db->quoteName('enabled'))
			->from($db->quoteName('#__extensions'))
			->where($db->quoteName('name') . ' = ' . $db->quote($parent->getManifest()->name));
		$db->setQuery($query);

		try
		{
			$db->execute();
			$enabled = $db->loadResult();
		}
		catch (RuntimeException $e)
		{
			$enabled = false;
		}

		// Fix update servers.
		if (version_compare($this->version, '1.3.0', 'lt'))
		{
			$query = $db->getQuery(true)
				->delete($db->quoteName('#__update_sites'))
				->where($db->quoteName('location') . ' = ' . $db->quote('http://www.hosting-skills.lu/updates/hsfirephp.xml'));
			$db->setQuery($query);
			$db->execute();
		}

		$parent->getParent()->message = sprintf(Text::_('PLG_SYSTEM_HSFIREPHP_UPDATE'), $this->version) . ($enabled ? '' : '<br /><br /><span style="color: red;">' . Text::_('PLG_SYSTEM_HSFIREPHP_CONFIGURE')) . '</span><br /><br /><img src="../plugins/system/hsfirephp/hsfirephp.png" alt="HSFirePHP!" /><br /><br />';

		return true;
	}

	/**
	 * This method is called after a component is uninstalled.
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.3.0
	 */
	public function uninstall(InstallerAdapter $parent): bool
	{
		return true;
	}

	/**
	 * Function called after extension installation/update/removal procedure finishes
	 *
	 * @param   string            $type    The type of change (install, update, discover_install or uninstall)
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 *
	 * @since   1.3.0
	 */
	public function postflight(string $type, InstallerAdapter $parent): bool
	{
		return true;
	}
}